#!/usr/bin/perl
#
# script used to analyze the output of the logging function
# in atemu

use warnings;

# Global variables
my @callStack;                # call stacks for the chips
my @functions;                # array of function addresses to names hashes, from nm
#my $graphchips;
my $showcallinfo = 0;
my $tempfn = "junkblah293.tmp";
my $execfile;
my %temphash;

# Argument Processing
if ($ARGV[0] =~ /help/) {
    print "Call Stack & dump creator\n";
    print "Generates Call stack information for atemu log files logged with dump and call info\n";
    print "Requires log files to be logged with -DC\n";
    print "usage:\tcallstack [inputfile] [chip 0 executable] [chip 1 executable] ... [chip n executable]\n";
    print "Output is to <inputfilename root>.dump\n";
    exit(0);
}

my $file;
if ($#ARGV >= 0) {
    $file = $ARGV[0];
    die unless open(LOGFILE, "bzcat $file |");
} else {
    print "Please specify a input log file\nUse 'callchain --help' for usage info\n";
    exit;
}

$i = 1;
# get symbol information
while ($i <= $#ARGV) {
    die "Can't open executable file $ARGV[$i]\n" 
	unless open(TESTREAD, "< $ARGV[$i]");
    close TESTREAD;

    system("nm -n $ARGV[$i] > $tempfn");
    die "Can't open symbol output file; probably didn't execute nm correctly or insufficient write privleges\n" 
	unless open(SYMBOL, "< $tempfn");
    
    undef %temphash;
    my %temphash;
    while (<SYMBOL>) {
	@words = split;
	$temphash{ hex($words[0]) } = $words[2];
    }
    $j = $i - 1;
    $functions[$j] = { %temphash };
    close SYMBOL;

    $i++;
    system("rm -f $tempfn\n");
}

########### Open Dump file, use root of the log file name
@temp = split /\./, $file;
$outfilename = $temp[0];
$outfilename .= ".dump";
die unless open(DUMPFILE, ">$outfilename");

print "Processing logfile...\n";
while (<LOGFILE>) {
    
    @words = split;

    $chipno = $words[0];
    $cycles = $words[2];
    if ($words[1] =~ /DUMP/) {
	if ($#words == 37) { 
#	    print DUMPFILE "Chip $words[0] - cycle $cycles => pc: 0x$words[3] sp: 0x$words[4] sreg: $words[5]\n\t";
	    print DUMPFILE "DUMP\n$words[0]\n$cycles\n$words[3]\n$words[4]\n$words[5]\n";
	    for ($i = 0; $i < 32; $i++) {
		$offset = $i + 6;
#		print DUMPFILE "reg$i: $words[$offset]";
		print DUMPFILE "$words[$offset] ";
#		if ($i % 4 == 3) {
#		    print DUMPFILE "\n\t";
#		}
	    }
	    print DUMPFILE "\n";

#	    print DUMPFILE "Call Stack:\n";
	    $listref = $callStack[$chipno];
	    for ($i = 0; $i <= $#{ $listref }; $i++) {
		$funcaddr = $listref->[$i];
		%function = %{ $functions[$chipno] };
		if ($funcaddr > 32) {
		    if (defined $function{$funcaddr}) {
			print DUMPFILE "$function{$funcaddr}\n";
		    } else {
			print DUMPFILE "0x$funcaddr\n";
		    }
		} else {
		    print DUMPFILE "INT $funcaddr\n";
		}
	    }
	}
    }
    elsif ($words[1] =~ /INTR/) {
	push(@{$callStack[$chipno]}, $words[4]);
    }
    elsif ($words[1] =~ /RETI/) {
	pop @{$callStack[$chipno]};
    }
    elsif ($words[1] =~ /CALL/ || $words[1] =~ /TASK/) {
	push(@{$callStack[$chipno]}, hex($words[4]) * 2);
    }
    elsif ($words[1] =~ /RETC/) {         # assembly: RET
	pop @{$callStack[$chipno]};
    }

}

print "Output saved to $outfilename\n";
