/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef XATDB_ATEMU_H
#define XATDB_ATEMU_H

#include <inttypes.h>
#include <sys/types.h>

extern pid_t child_pid;

/* this doesn't really go here, but.... */
/* flag that we want to exit NOW, for breaking out of loops */
extern int xatdb_quit;
extern int chips_destroyed;
extern int atemu_load_finished;

/* to (help) prevent pipe-related deadlock */
extern int pipe_write_safe;

/* idle processor for async commands */
int atemu_idle_process(void *data);

/********************************************/
/* these functions are global to all chips  */
/********************************************/

/* start an emulator with the given command line */
/* returns < 0 on failure */
int atemu_init( const char * file );

/* starts continuous execution of the emulator */
void atemu_go();

/* single opcode execution of the emulator */
void atemu_step();

/* interrupts execution of the emulator.  Does not quit it */
void atemu_stop();

/* makes the child process exit */
void atemu_quit();

/* this gets queued up after the chip unassemblies to mark the end of loading */
void atemu_load_finish();

/********************************************/
/* these functions are specific to one chip */
/********************************************/

/* add watchpoint */
void atemu_add_watch(int chipid, uint32_t addr, int size);

/* add breakpoint */
void atemu_add_break(int chipid, uint32_t addr);

/* add notifypoint */
void atemu_add_notify(int chipid, uint32_t addr, uint32_t id);

/* clear watchpoint */
void atemu_clear_watch(int chipid, uint32_t addr);

/* clear breakpoint */
void atemu_clear_break(int chipid, uint32_t addr);

/* clear notifypoint */
void atemu_clear_notify(int chipid, uint32_t addr);

/* dump register values */
void atemu_dump_regs(int chipid );

/* dump sram values */
void atemu_dump_sram(int chipid, long int addr );

/* assembly dump of chip flash */
void atemu_unassemble(int chipid);

/* mark the status bar as finished */
void atemu_status_bar_finish();

#endif
