/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef XATDB_BREAKPOINTS_H
#define XATDB_BREAKPOINTS_H

extern int auto_breaks_active;

/* functions for using breakpoints */
void add_breakpoint(int chipid, long addr);

void remove_all_breakpoints(int chipid);
void remove_breakpoint_by_addr(int chipid, long addr);
void remove_breakpoint_by_c_line(int chipid, char *cfile, long line);

/* auto breakpoint manipulation (for C stepping) */
void generate_auto_breakpoints(int chipid);
void activate_auto_breakpoints(int chipid);
void deactivate_auto_breakpoints();

/* return true if there is a breakpoint at the address */
int is_breakpoint_at_addr(int chipid, long addr);
int is_breakpoint_at_c_line(int chipid, char *cfile, long line);

/* tables of breakpoints stuff */
int asm_break_comp(const void *a, const void *b);
int source_break_comp(const void *a, const void *b);
int auto_break_comp(const void *a, const void *b);

typedef struct _asm_break_tuple{
    long addr;
}asm_break_tuple;

typedef struct _source_break_tuple{
    long addr;
}source_break_tuple;

typedef struct _auto_break_tuple{
    long addr;
}auto_break_tuple;

#endif

