/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef CHIP_DATA_H
#define CHIP_DATA_H

#include <gtk/gtk.h>
#include <inttypes.h>

#include "table.h"
#include "func_data.h"
#include "type_data.h"

/* a structure to store text buffers and list stores */
/*     for each chip. */

typedef struct _chip_data chip_data;

/* total number of chips */
extern int chip_array_cnt;
extern int curr_chip;
extern chip_data *chip_array;

/* default size to guess a variable is */
#define XATDB_DEFAULT_VAR_SIZE 1

/* to map from asm addr to c line */
struct tuple_file_addr {
    long int addr_begin;
    long int addr_end;
    long int line;
    char * fname;
};

/* to map from asm addr to texbuffer line */
struct tuple_line_addr {
    long int addr;
    long int line;
};

/* to map from C file name to textbuffer and menu index */
struct tuple_cfile_textbuffer{
    char *fname;
    GtkTextBuffer *textbuff;
    GtkWidget *textview;
    int menu_index;
    GtkWidget *menu_item;
};

/* to hold global symbol information */
struct tuple_global_addr {
    char *name;

    struct type_data *td;
    int type_style;

    long int addr;

    int notify_id;

    int notify_child_start;
    int notify_child_end;

    uint8_t *value;
    GtkTreeIter iter;
};

enum {
    REGS_TAB_X = 32,
    REGS_TAB_Y,
    REGS_TAB_Z,
    REGS_TAB_PC,
    REGS_TAB_STACK,
    REGS_TAB_SREG,
};

/* REGS tree model enumerations */
enum {
    REGS_NAME = 0,
    REGS_HEX,
    REGS_DEC,
};

/* STACK tree model enumerations */
enum {
    STACK_SFILE = 0,
    STACK_SLINE,
};

/* LOCALS tree model enumerations */
enum {
    LOCALS_NAME = 0,
    LOCALS_DATA,
    LOCALS_ID,
};

/* GLOBALS tree model enumerations */
enum {
    GLOBALS_ICON = 0,
    GLOBALS_NAME,
    GLOBALS_DATA,
    GLOBALS_ID,
};

/* BREAK tree model enumerations */
enum {
    BREAK_ADDR_TEXT = 0,
    BREAK_LINE,
    BREAK_ADDR,
};

/* WATCH tree model enumeration */
enum {
    WATCH_LOC = 0,
    WATCH_NAME,
    WATCH_ADDR,
    WATCH_SIZE,
    WATCH_ID,
};

/* insert mappings into the appropriate tables */
void add_addr_file_mapping( int chipid, long int begin, long int end,
    long int line, char * file );
void add_addr_line_mapping( int chipid, long int addr, long int line );
void add_func_data( int chip_id, struct func_data *fd );
void add_type_data( int chipid, struct type_data *td );
void add_cfile(int chipid, char *fname);
void make_cfile_menu(int chipnum);
void make_globals_list(int chipnum);
void render_global( int chipid, struct tuple_global_addr *glob );
void update_global( int chipid, int id, long int addr, uint8_t new_value );
void size_global( int chipid, char *name, struct type_data *td );
GtkTreeIter add_global( int chipid, char * symbol, long int addr, GtkTreeIter *parent );

/* manipulations of watch/breakpoint lists */
void move_selection( GtkTreeIter iter, GtkTreeView *tv );

/* look up from address mapping tables */
int address_to_text_line( int chipnum, long int address );
long text_line_to_address(int chipnum, int line);
long address_to_c_line( int chipnum, long int address, char **cfile );
long c_line_to_address(int chipnum, char *cfile, int line);
/* returns the addresses that a c line covers, or (-1, -1) if invalid */
void c_line_to_address_range( int chipnum, char *cfile, int line, long *start, long *end );
char * address_to_function( int chipnum, long int address );
struct func_data * address_to_func_data( int chipnum, long int address );
struct tuple_global_addr *addr_to_global_data( int chipid, long int addr );

/* look up things via cfile index */
GtkTextBuffer *cfile_index_to_textbuffer(int chipnum, int index);
GtkWidget *cfile_index_to_textview(int chipnum, int index);
GtkWidget *cfile_index_to_menu_item(int chipnum, int index );
char *cfile_index_to_cfile(int chipnum, int index);

/* look things up via cfile name */
int cfile_to_cfile_index(int chipnum, char *fname);
GtkTextBuffer *cfile_to_textbuffer(int chipnum, char *fname);
GtkWidget *cfile_to_textview(int chipnum, char *fname);

/* look things up via debugging info type id string */
struct type_data *id_to_type_data( int chipnum, char *id );
struct local *id_to_local_data( int chipid, int id );
struct tuple_global_addr *id_to_global_data( int chipid, int id );

/* create and remove chip structures */
void create_all_chips( int ntab );
void destroy_chips();

/* parses output from the emulator */
void process_buffer( char *buff );

/* set values in register display */
void set_register( int chipn, int line, long int value, int h_field, int d_field );
void refresh_registers( int chipn, long data_addr, uint8_t buff[16] );

/* updating locals and the register display */
void render_local( int chipid, struct local *loc, GtkTreeIter *iter );
void update_locals( int chipid, long int addr );
void new_locals( int chipid, long int addr, uint8_t buff[16] );

/* a comparison function to help sort the TreeView alphabetically */
gint global_iter_compare( GtkTreeModel *model, GtkTreeIter *a,
    GtkTreeIter *b, gpointer user_data);

/* a comparison function for sorting two source files
   represented by char* */
int cfile_string_compare( gconstpointer a, gconstpointer b );
    
/* updating stack display */
void add_stack_item( int chipid, long int addr );

#endif
