/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <strings.h>
#include <sys/stat.h>

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "atemu.h"
#include "debug.h"
#include "version.h"

static void signal_handler(int type){
    switch(type) {
        case SIGCHLD:
            wait( NULL );
            child_pid = 0;
            break;
        case SIGINT:
        case SIGTERM:
            if(child_pid != 0){
                kill(child_pid, SIGTERM);
                wait(NULL);
            }
            exit(-1);
            break;
        default:
            break;
    }

    signal(type, signal_handler);
}

int main (int argc, char *argv[], char *envp[]){
    GtkWidget *window1;
    int index;
    const char *usage_string;
    
    gtk_set_locale();
    gtk_init(&argc, &argv);
    usage_string = _("Usage: xatdb [options] [file path]");

    /* Read any command line args */
    for( index = 1; index < argc; index++ ) {
        
        /* Place additional argument handlers here at the top of the loop */
        
        if( !strcasecmp( _("--help"), argv[index] ) || !strcmp( _("-h"), argv[index] ) ) {
            printf( _("%s\n%s\n%s\n%s\n%s\n%s\n\n"),
                usage_string,
                _("Options:"),
                _("  -h, --help\t\t\tPrint this help message"),
                _("  -s, --no-messages\t\tSuppress error messages"),
                _("  -v, --verbose\t\t\tDisplay additional information"),
                _("  -V, --version\t\t\tDisplay version and copyright")
            );
            return 0;
        } else if( !strcasecmp( _("--no-messages"), argv[index] ) || !strcmp( _("-s"), argv[index] ) ) {
            verbose_level = 0;
        } else if( !strcasecmp( _("--verbose"), argv[index] ) || !strcmp( _("-v"), argv[index] ) ) {
            verbose_level = 2;
        } else if( !strcasecmp( _("--version"), argv[index] ) || !strcmp( _("-V"), argv[index] ) ) {
            printf( _("%s\n\n%s\n%s\n%s\n\n"),
                XATDB_VERSION,
                _("Copyright (c) 2003, University of Maryland"),
                _("Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks"),
                _("All rights reserved. See the source for copying conditions.")
            );
            return 0;
        } else if( *(argv[index]) == '-' ) {
            /* Handle invalid arguments */
            printf( _("xatdb: invalid option -- %s\n"), argv[index] );
            return 1;
        } else if( argc - index > 1 ) {
            /* Handle otherwise invalid or out of order arguments */
            printf( "%s\n\n", usage_string );
            return 1;
        } else {
            atemu_init( argv[index] );
        }
    }
    
    /* create the main window */
    window1 = create_window1();
    
    gtk_widget_show_all(window1);

    g_idle_add_full(G_PRIORITY_LOW, atemu_idle_process, NULL, NULL);

    signal( SIGCHLD, signal_handler );
    signal( SIGINT, signal_handler );
    signal( SIGTERM, signal_handler );
    
    gtk_main();

    return 0;
}

