/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef TYPE_DATA_H
#define TYPE_DATA_H

#include <glib.h>
#include <inttypes.h>

enum td_type {
    UNKNOWN,
    SIGNED_NUM,
    UNSIGNED_NUM,
    STRUCT,
    ENUM,
    ARRAY,
    POINTER,
    FUNCTION,
    REFERENCE,
    S_REFERENCE,
};

struct field {
    char *name;

    struct type_data *type;

    /* to be used by STRUCT fields only */
    uint16_t bit_offset;
    uint16_t bit_length;

    /* to be used by ENUM fields only */
    uint16_t value;
};

struct type_data {
    char *name;
    char *id;

    enum td_type type;

    long long int begin, end;
    uint16_t size;

    GSList *fields;

    /* for references, if you are the root, point to yourself */
    struct type_data * root;
    /* for references, used to create linked list back to root, useful for naming a type */
    struct type_data * reference;

    /* for pointers, this is the pointed to type */
    /* for arrays, this is the array element type */
    /* for functions, this is the return type */
    struct type_data * extra;
};

struct type_data *create_type_data( char *name, char *id, uint16_t size,
    long long int begin, long long int end, enum td_type type );

void add_field_to_type( struct type_data * td, struct field * f );
int type_data_compare_id( const void * a, const void * b );
void free_type_data( void * a );

struct field * create_field( char * name, struct type_data *td, 
    uint16_t bit_off, uint16_t bit_len, uint16_t val );
void free_field( struct field * f );

int render_type( char *buff, int size, uint8_t *data, struct type_data *td );

#endif
